/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.util.*;
import java.io.*;

class RemoteSyncEvent 
    extends EventObject implements Serializable
{
    ArrayList sketchPanelStates;
    ArrayList controlPanelStates;

    String status;
    String errmsg;

    public RemoteSyncEvent()
    {
	super(new Object()); //hack
	sketchPanelStates = null;
	controlPanelStates = null;
	status = null;
	errmsg = null;
    }

    public String toString()
    {
	String result = "";
	Iterator it;
	if(sketchPanelStates != null) {
	    result += "SketchPanelStates: ";
	    it = sketchPanelStates.iterator();
	    while(it.hasNext()) 
		result += it.next()+" ";
	    result += "\n";
	}
	if(controlPanelStates != null) {
	    result += "ControlPanelStates: ";
	    it = controlPanelStates.iterator();
	    while(it.hasNext()) 
		result += it.next()+" ";
	    result += "\n";
	}
	if(status != null)
	    result += "Status: "+status + "\n";

	if(errmsg != null)
	    result += "ErrMsg: "+errmsg + "\n";

	return result;
    }

    public static void main(String[] args) 
	throws IOException, ClassNotFoundException
    {
	
	RemoteSyncEvent foo = new RemoteSyncEvent();
	foo.sketchPanelStates.add(new String("foobar"));
	foo.sketchPanelStates.add(new String("foobar 2"));
	foo.controlPanelStates.add(new String("bar"));
	
	FileOutputStream ostream = new FileOutputStream("t.tmp");
	ObjectOutputStream p = new ObjectOutputStream(ostream);

	p.writeObject(foo);
	p.flush();
	ostream.close();

	System.out.println("deserialization");
	
	FileInputStream istream = new FileInputStream("t.tmp");
	ObjectInputStream ip = new ObjectInputStream(istream);

	RemoteSyncEvent ifoo = (RemoteSyncEvent) ip.readObject();

	System.out.println(ifoo);
	istream.close();
 
    }

    public void update(RemoteSyncEvent e)
    {
	if (e.sketchPanelStates != null)
	    sketchPanelStates = e.sketchPanelStates;
	if (e.controlPanelStates != null)
	    controlPanelStates = e.controlPanelStates;
	if (e.status != null)
	    status = e.status;
	if (e.errmsg != null)
	    errmsg = e.errmsg;
	//System.out.println("Update: "+this);
    }

    public Object clone()
    {
	RemoteSyncEvent result = new RemoteSyncEvent();
	result.sketchPanelStates = sketchPanelStates;
	result.controlPanelStates = controlPanelStates;
	result.status = status;
	result.errmsg = errmsg;
	return result;
    }
}
