/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a Distance Estimate.
 *
 * @author Allen Miu
 */
class DistanceEstimateParseHandlerMap extends ParseHandlerMap
{
    public static DistanceEstimateParseHandlerMap instance = 
	new DistanceEstimateParseHandlerMap();

    protected DistanceEstimateParseHandlerMap()
    {
	super();
	register("space", "check_beaconrec", "parse_space");
	register("id", "check_beaconrec", "parse_id");
	register("dist", "check_beaconrec", "parse_dist");
	register("dist_stat", "check_beaconrec", "parse_dist_stat");
	register("pos", "check_beaconrec", "parse_pos");
	register("last_update", "check_beaconrec", "parse_last_update");
    }

    public boolean check_beaconrec(Object o)
    {
	return o instanceof BeaconRecord;
    }

    public void parse_space(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.space = value;
	c.setUniqueName();
    }

    public void parse_id(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.id = Integer.valueOf(value);
	c.setUniqueName();
    }
	
    public void parse_dist(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.distStat = new DistanceStat();
	c.distStat.dist = Util.parseDouble(value);
    }
    
    public void parse_dist_stat(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.distStat = new DistanceStat();
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) {
	    Field.parseField(c.distStat, 
			     sr,
			     StatParseHandlerMap.instance);
	}
    }
	
    public void parse_pos(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.pos = new Position(-1, -1, -1);
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) {
	    Field.parseField(c.pos, 
			     sr,
			     PosParseHandlerMap.instance);
	}
    }

    public void parse_last_update(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.lastUpdate = Long.valueOf(value);
    }
}
