/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.orient;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.reader.Beacon;
import java.util.*;

/**
 * The data type that is passed to higher layer module that registers a
 * callback with the Orient module.
 *
 * @author Allen Miu
 */
public class OrientData extends Data
{
    private static final int MAX_SEQNUM = 65536;
    private static int globalSeqNum = 0;
    public int seqNum;

    public Float value;
    
    public OrientData(Beacon b, Float angle)
    {
	super(b);
	seqNum = globalSeqNum;
	globalSeqNum++;             // ensures unique sequence number
	globalSeqNum %= MAX_SEQNUM; // wrap around
	value = angle;
    }

    public interface OrientDataHandler {
	public void processOrientData(OrientData newData, BitSet fieldMask);
    }

    public void process(Object handler, BitSet fieldMask) 
    {
	Util.assert_nonjava(handler instanceof OrientDataHandler,
		    "OrientData::process() incorrect handler type");
	OrientDataHandler h = (OrientDataHandler) handler;
	h.processOrientData(this, fieldMask);
    }
    
    public String toString()
    {
	return value.toString();
    }
}

