/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import cricketdaemon.reader.CartesianCoord;
import java.util.*;

/**
 * A FieldParser that parses the input bytes as a CartesianCoordinate.
 *
 * @author Allen Miu
 */
public class FieldParser_CartesianCoord implements FieldParser
{
    public static FieldParser instance = 
	new FieldParser_CartesianCoord();

    private FieldParser_CartesianCoord()
    {
    }

    public static final int VALUELENGTH = 4;
    public static final int EXPECTED_FIELDLENGTH = VALUELENGTH*3;

    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length != EXPECTED_FIELDLENGTH)
	    Util.warn("FieldParser_CartesianCoord: length is invalid "+length);

	FieldParser floatParser = FieldParser_Float.instance;

	Float X = 
	    (Float) floatParser.parse(beacon, 0, VALUELENGTH);
	Float Y = 
	    (Float) floatParser.parse(beacon, VALUELENGTH, VALUELENGTH);
	Float Z = 
	    (Float) floatParser.parse(beacon, 2*VALUELENGTH, VALUELENGTH);

	return new CartesianCoord(X.floatValue(), Y.floatValue(), Z.floatValue());
    }

    public byte[] unparse(Object o) 
    {
	CartesianCoord coord = (CartesianCoord) o;
	byte[] result = new byte[EXPECTED_FIELDLENGTH];
	Float x = new Float(coord.x);
	Float y = new Float(coord.y);
	Float z = new Float(coord.z);

	FieldParser floatParser = FieldParser_Float.instance;
	byte[] temp = floatParser.unparse(x);
	Util.bufcpy(result, temp, 0, VALUELENGTH);
	temp = floatParser.unparse(y);
	Util.bufcpy(result, temp, VALUELENGTH, VALUELENGTH);
	temp = floatParser.unparse(z);
	Util.bufcpy(result, temp, 2*VALUELENGTH, VALUELENGTH);

	return result;
    }

    public Object stringToReturnType(String s) 
    {
	return CartesianCoord.parseCoord(s);
    }

    public Object GPSDStringToReturnType(String s) 
    {
	return stringToReturnType(s);
    }

    public static void main(String[] args)
    {
	FieldParser p = FieldParser_CartesianCoord.instance;
	byte[] r = p.unparse(new CartesianCoord(4.5, 4660, -1));

	for(int i = 0; i < r.length; i++) 
	    System.out.println(i+" "+Integer.toHexString(r[i]));
	
	System.out.println(p.parse(r, 0, EXPECTED_FIELDLENGTH));
    }
}
