/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import cricketdaemon.CricketDaemon;

/**
 * A FieldParser that parses the input bytes as an Integer.
 *
 * @author Allen Miu
 */
public class FieldParser_Long implements FieldParser
{
    public static FieldParser instance = new FieldParser_Long();

    private FieldParser_Long()
    {
    }

    // Byte order: LSB ... MSB
    /*
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Long: length is invalid "+length);

	int result = normalize(beacon[offset]);
	
	if(length > 1)
	    result += normalize(beacon[offset+1]) * 256;      // 2^8
	if(length > 2)
	    result += normalize(beacon[offset+2]) * 65536;    // 2^16
	if(length > 3)
	    result += normalize(beacon[offset+3]) * 16777216; // 2^24
	
	//System.out.println("FieldParser_Long "+result);

	return new Long(result);
    }
    */
    // MSB ... LSB
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
	    if(length < 1 || length > 8)
		Util.warn("FieldParser_Long: length is invalid "+length);
	    
	    //int result = normalize(beacon[offset]);
	    int result = 0;

	    for(int i = 0; i < length; i++) 
		result = normalize(beacon[offset+i]) + result*256;
	    
	    //System.out.println("FieldParser_Long "+result);
	    
	    return new Long(result);
	}
	else {
	    return Long.valueOf(new String(beacon, offset, length));
	}

    }

    // WARNING: assumes positive integer only !!!!
    public byte[] unparse(Object o)
    {
	long i = ((Long) o).longValue();

	if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
	    byte[] result = new byte[4];
	    
	    for(int c = 3; c >= 0; c--) {
		result[c] = (byte) (i % 256);
		i -= normalize(result[c]);
		i /= 256;  // right shift
		if( i <= 0 )
		    break;
	    }
	    
	    return result;
	}
	else {
	    return String.valueOf(i).getBytes();
	}
    }

    public Object stringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    result = Long.valueOf(s);
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    public Object GPSDStringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
		result = Long.valueOf(s, 16); // expects hex
	    }
	    else {
		result = Long.valueOf(s); // expects decimal
	    }
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    private int normalize(byte b) {
	if( b < 0 )
	    return b + 256;
	else
	    return b;
    }

    public static void main(String[] args) {
	CricketDaemon.listenerVersion = CricketDaemon.CRICKETV2;
	FieldParser p = FieldParser_Long.instance;
	Long v = new Long(1087268367801l);
	byte[] r = p.unparse(v);
	//byte[] r = p.unparse(new Long(152));
	for(int i = 0; i < r.length; i++)
	    System.out.println(i+" "+Long.toHexString(r[i]));

	Long parsed = (Long) p.parse(r, 0, r.length);
	//System.out.println(Long.toHexString(parsed.intValue()));
	System.out.print("correct: "+v+"\ntested:  ");
	System.out.println(Long.toString(parsed.longValue()));
    }
}
