/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.space;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.reader.Beacon;
import java.util.*;

/**
 * The data type that is passed to higher layer module that registers a
 * callback with the Space module.
 *
 * @author Allen Miu
 */
public class SpaceData extends Data
{
    private static final int MAX_SEQNUM = 65536;
    private static int globalSeqNum = 0;
    public int seqNum;

    DistanceEstimate[][] values;
    
    public SpaceData(Beacon b)
    {
	super(b);
	seqNum = globalSeqNum;
	globalSeqNum++;             // ensures unique sequence number
	globalSeqNum %= MAX_SEQNUM; // wrap around
	values = new DistanceEstimate[3][]; // zero is null
    }

    public interface SpaceDataHandler {
	public void processSpaceData(SpaceData newData, BitSet fieldMask);
    }

    public void process(Object handler, BitSet fieldMask) 
    {
	Util.assert_nonjava(handler instanceof SpaceDataHandler,
		    "SpaceData::process() incorrect handler type");
	SpaceDataHandler h = (SpaceDataHandler) handler;
	h.processSpaceData(this, fieldMask);
    }
    
    public void insert(DistanceEstimate[] value, int field)
    {
	if(field < 0 || field >= values.length || values[field] != null) {
	    // if value already exists, do a null operation to
	    // prevent modules from tampering with beacon data.
	    Util.warn("SpaceData::insertValue() seqNum="+seqNum+
		      " values["+field+"] silently ignored");
	    return;
	}
	values[field] = value;
    }

    public DistanceEstimate[] getMean()
    {
	return values[SpaceModule.MEAN];
    }
    
    public DistanceEstimate[] getMode()
    {
	return values[SpaceModule.MODE];
    }
    
    public String toString()
    {
	String result = "SpaceData:\n";
	if(values[SpaceModule.MEAN] != null) {
	    result += "  List of sorted mean Beacon Distances:  ";
	    for(int i = 0; i < values[SpaceModule.MEAN].length; i++)
		result += "("+values[SpaceModule.MEAN][i]+") ";
	}
	result += "\n";
	if(values[SpaceModule.MODE] != null) {
	    result += "  List of sorted mode Beacon Distances:  ";
	    for(int i = 0; i < values[SpaceModule.MODE].length; i++)
		result += "("+values[SpaceModule.MODE][i]+") ";
	}
	return result;
    }
}

